@@ -336,10 +336,6 @@ class Datacube(object):
         """
         Find datasets for a product.
 
-        .. note:
-            This is a lower level function than most people will use. See :meth:`load` for
-            a simpler to use function.
-
         :param kwargs: see :class:`datacube.api.query.Query`
         :return: list of datasets
         :rtype: list[:class:`datacube.model.Dataset`]
@@ -409,7 +405,8 @@ class Datacube(object):
             list of measurement dicts with keys: {'name', 'dtype', 'nodata', 'units'}
 
         :param data_func:
-            function to fill the data
+            function to fill the storage with data. It is called once for each measurement, with the measurement
+            as an argument. It should return an appropriately shaped numpy array.
 
         :rtype: :class:`xarray.Dataset`
 
