@@ -72,10 +72,10 @@ class Dataset(object):
 
     :type type_: DatasetType
     :param dict metadata_doc: the document (typically a parsed json/yaml)
-    :param str local_uri: A URI to access this dataset locally.
+    :param list[str] uris: All active uris for the dataset
     """
 
-    def __init__(self, type_, metadata_doc, local_uri, sources=None,
+    def __init__(self, type_, metadata_doc, local_uri=None, uris=None, sources=None,
                  indexed_by=None, indexed_time=None, archived_time=None):
         assert isinstance(type_, DatasetType)
 
@@ -87,11 +87,18 @@ class Dataset(object):
         #: :type: dict
         self.metadata_doc = metadata_doc
 
-        #: The most recent local file available to access the data, if any.
-        #: Note that a dataset can have multiple uris, not all of which are local files.
-        #: To get all uris for a dataset, use index.datasets.get_locations()
-        #: :type: str
-        self.local_uri = local_uri
+        if local_uri:
+            warnings.warn(
+                "Dataset.local_uri has been replaced with list Dataset.uris",
+                DeprecationWarning
+            )
+            if not uris:
+                uris = []
+
+            uris.append(local_uri)
+
+        #: Active URIs in order from newest to oldest
+        self.uris = uris
 
         #: The datasets that this dataset is derived from (if requested on load).
         #: :type: dict[str, Dataset]
@@ -115,6 +122,18 @@ class Dataset(object):
     def metadata_type(self):
         return self.type.metadata_type if self.type else None
 
+    @property
+    def local_uri(self):
+        """
+        The latest local file uri, if any.
+        :rtype: str
+        """
+        local_uris = [uri for uri in self.uris if uri.startswith('file:')]
+        if local_uris:
+            return local_uris[0]
+
+        return None
+
     @property
     def local_path(self):
         """
@@ -197,22 +216,33 @@ class Dataset(object):
         :rtype: geometry.CRS
         """
         projection = self.metadata.grid_spatial
+        if not projection:
+            return None
 
         crs = projection.get('spatial_reference', None)
         if crs:
             return geometry.CRS(str(crs))
 
-        # TODO: really need CRS specified properly in agdc-metadata.yaml
-        if projection['datum'] == 'GDA94':
-            return geometry.CRS('EPSG:283' + str(abs(projection['zone'])))
+        # Try to infer CRS
+        zone_ = projection.get('zone')
+        datum_ = projection.get('datum')
+        if zone_ and datum_:
+            try:
+                # TODO: really need CRS specified properly in agdc-metadata.yaml
+                if datum_ == 'GDA94':
+                    return geometry.CRS('EPSG:283' + str(abs(zone_)))
+                if datum_ == 'WGS84':
+                    if zone_[-1] == 'S':
+                        return geometry.CRS('EPSG:327' + str(abs(int(zone_[:-1]))))
+                    else:
+                        return geometry.CRS('EPSG:326' + str(abs(int(zone_[:-1]))))
+            except geometry.InvalidCRSError:
+                # We still return None, as they didn't specify a CRS explicitly...
+                _LOG.warning(
+                    "Can't figure out projection: possibly invalid zone (%r) for datum (%r).", zone_, datum_
+                )
 
-        if projection['datum'] == 'WGS84':
-            if projection['zone'][-1] == 'S':
-                return geometry.CRS('EPSG:327' + str(abs(int(projection['zone'][:-1]))))
-            else:
-                return geometry.CRS('EPSG:326' + str(abs(int(projection['zone'][:-1]))))
-
-        raise RuntimeError('Cant figure out the projection: %s %s' % (projection['datum'], projection['zone']))
+        return None
 
     @cached_property
     def extent(self):
@@ -223,14 +253,24 @@ class Dataset(object):
         def xytuple(obj):
             return obj['x'], obj['y']
 
+        # If no projection or crs, they have no extent.
         projection = self.metadata.grid_spatial
+        if not projection:
+            return None
+        crs = self.crs
+        if not crs:
+            _LOG.debug("No CRS, assuming no extent (dataset %s)", self.id)
+            return None
 
-        if 'valid_data' in projection:
-            return geometry.Geometry(projection['valid_data'], crs=self.crs)
-        else:
-            geo_ref_points = projection['geo_ref_points']
+        valid_data = projection.get('valid_data')
+        geo_ref_points = projection.get('geo_ref_points')
+        if valid_data:
+            return geometry.Geometry(valid_data, crs=crs)
+        elif geo_ref_points:
             return geometry.polygon([xytuple(geo_ref_points[key]) for key in ('ll', 'ul', 'ur', 'lr', 'll')],
-                                    crs=self.crs)
+                                    crs=crs)
+
+        return None
 
     def __eq__(self, other):
         return self.id == other.id
